use crate::services::confirm::ConfirmService;
use crate::services::rest::RestService;
use common::Achievement;
use common::DeleteAchievement;
use common::ToggleAchievement;
use common::UpdateAchievementTimeOfReveal;
use std::ops::Deref;
use wasm_bindgen::JsCast;
use wasm_bindgen_futures::spawn_local;
use yew::classes;
use yew::function_component;
use yew::html;
use yew::use_state;
use yew::Callback;
use yew::Html;
use yew::Properties;

#[derive(Properties, PartialEq)]
pub struct Props {
    pub achievement: Achievement,
    pub number: usize,
}

#[function_component]
pub fn AchievementRevealTime(props: &Props) -> Html {
    let achievement = &props.achievement;
    let uuid = achievement.uuid;

    let time_of_reveal: Option<String> = achievement
        .time_of_reveal
        .map(|naive_time| naive_time.format("%H:%M").to_string());

    let timed_reveal_enabled = use_state(|| time_of_reveal.is_some());
    let input_time = use_state(|| time_of_reveal.clone().unwrap_or("".to_string()));
    let awaiting_response = use_state(|| false);

    let onsubmit = {
        let awaiting_response = awaiting_response.clone();
        let timed_reveal_enabled = timed_reveal_enabled.clone();
        let input_time = input_time.clone();
        Callback::from(move |e: web_sys::SubmitEvent| {
            e.prevent_default();

            let new_time_of_reveal = if *timed_reveal_enabled {
                if let Ok(naive_time) = chrono::NaiveTime::parse_from_str(&input_time, "%H:%M") {
                    Some(naive_time)
                } else {
                    // TODO: show UI error
                    log::debug!("Could not parse time: {}", *input_time);
                    return;
                }
            } else {
                None
            };

            let payload = UpdateAchievementTimeOfReveal {
                time_of_reveal: new_time_of_reveal,
                uuid,
            };
            awaiting_response.set(true);
            let awaiting_response = awaiting_response.clone();
            spawn_local(async move {
                let res = RestService::update_time_of_reveal(payload).await;
                awaiting_response.set(false);
            });
        })
    };

    let oninput_time = {
        let input_time = input_time.clone();
        Callback::from(move |e: web_sys::InputEvent| {
            let Some(input) = e
                    .target()
                    .and_then(|t| t.dyn_into::<web_sys::HtmlInputElement>().ok()) else { unreachable!() };
            log::debug!("{:?}", input.value());
            input_time.set(input.value());
        })
    };

    let oninput_timed_reveal_checkbox = {
        let timed_reveal_enabled = timed_reveal_enabled.clone();
        Callback::from(move |e: web_sys::InputEvent| {
            let Some(input) = e
                    .target()
                    .and_then(|t| t.dyn_into::<web_sys::HtmlInputElement>().ok()) else { unreachable!() };
            timed_reveal_enabled.set(input.checked());
        })
    };

    let new_value: Option<&str> = timed_reveal_enabled.then_some(&**input_time);
    let show_submit_button: bool = time_of_reveal.as_deref() != new_value;

    html! {
        <form {onsubmit}>

        <div class="row flex">
            // Achievement number
            <div class="flex-intrinsic-size">
                <p>{format!("{}.", props.number)}</p>
            </div>

            // Achievement text
            <div class="flex-grow">
                <p>{&achievement.goal}</p>
            </div>
        </div>

        // Enable timed reveal checkbox
        <label class="row">
          <input oninput={oninput_timed_reveal_checkbox} checked={*timed_reveal_enabled} type="checkbox" />
          <span class="label-body">{"Timed reveal"}</span>
        </label>

        { if *timed_reveal_enabled { html! {
            // Time input
            <div>
                <label for="revealTimeInput">{"Reveal time"}</label>
                <input oninput={oninput_time} value={(*input_time).clone()} type="time" id="revealTimeInput" />
            </div>
        }} else { html! {}}}

        // Submit button
        { if show_submit_button { html! {
            <input class="button-primary" type="submit" value="Submit" disabled={ *awaiting_response } />
        }} else { html! {}}}

        <hr />
        </form>
    }
}
